'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.sync = sync;
exports.files = files;
exports.request = request;

exports.default = function () {
  var _ref2 = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  var

  /**
   * base Path to webpack assets on disk.
   * dev Allow parent process to send new stats object via IPC.
   * serve True to serve webpack assets from publicPath, false otherwise.
   * stats Path to webpack stats JSON file.
   * @returns {Function} Middleware.
   */
  stats = _ref2.stats;
  var _ref2$base = _ref2.base;
  var base = _ref2$base === undefined ? stats ? _path2.default.dirname(stats) : null : _ref2$base;
  var _ref2$dev = _ref2.dev;
  var dev = _ref2$dev === undefined ? process.env.HAS_WEBPACK_ASSET_EVENTS : _ref2$dev;
  var _ref2$serve = _ref2.serve;
  var serve = _ref2$serve === undefined ? !dev && stats : _ref2$serve;

  var update = updater({ serve: serve, base: base });

  // Standard `webpack` output in production is just a single stats file, so
  // read that and use that for assets.
  if (stats) {
    update(JSON.parse((0, _fs.readFileSync)(stats, 'utf8')));
  }

  // `webpack-udev-server` provides events to the process when new assets have
  // been generated for the client portion of the code.
  if (dev) {
    process.on('webpack-stats', update);
  }

  if (serve) {
    return (0, _flowRight2.default)(files(), request());
  }

  return request();
};

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _keyBy = require('lodash/keyBy');

var _keyBy2 = _interopRequireDefault(_keyBy);

var _flowRight = require('lodash/flowRight');

var _flowRight2 = _interopRequireDefault(_flowRight);

var _webpackAssets = require('webpack-assets');

var _webpackAssets2 = _interopRequireDefault(_webpackAssets);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * [assets description]
 * @type {Array}
 */
var assets = [];

/**
 * A URL index for serving all the assets in the generated webpack stats.
 * This makes it very quick to serve a particular file since it's just a single
 * dictionary lookup.
 * @type {Object}
 */
var index = {};

/**
 * Given a webpack stats object, create a new set of assets.
 * @param {[type]} stats [description]
 * @returns {[type]}       [description]
 */
function updater(_ref) {
  var serve = _ref.serve;
  var base = _ref.base;

  if (serve) {
    return function (stats) {
      assets = (0, _webpackAssets2.default)(stats);
      sync(base);
    };
  }
  return function (stats) {
    assets = (0, _webpackAssets2.default)(stats);
  };
}

/**
 * Update the local asset data for serving.
 * @param {String} base Where on disk to find the assets.
 * @returns {[type]} [description]
 */
function sync(base) {
  assets.forEach(function (asset) {
    if (!asset.contents) {
      asset.contents = (0, _fs.readFileSync)(_path2.default.join(base, asset.file));
    }
  });
  index = (0, _keyBy2.default)(assets, function (asset) {
    return asset.url;
  });
}

/**
 * Serve assets generated by webpack.
 * @returns {Function} Herp.
 */
function files() {
  return function (app) {
    var _request = app.request;

    return _extends({}, app, {
      request: function request(req, res) {
        if (index[req.url]) {
          if (req.method !== 'GET' && req.method !== 'HEAD') {
            res.statusCode = 405;
            res.setHeader('Allow', 'GET, HEAD');
            res.setHeader('Content-Length', '0');
            res.end();
          } else {
            var asset = index[req.url];
            res.statusCode = 200;
            res.setHeader('ETag', asset.hash);
            res.setHeader('Content-Length', asset.contents.length);
            res.setHeader('Content-Type', asset.contentType);
            res.end(asset.contents);
          }
        } else {
          _request(req, res);
        }
      }
    });
  };
}

function request() {
  return function (app) {
    var _request2 = app.request;

    return _extends({}, app, {
      request: function request(req, res) {
        req.assets = assets;
        _request2(req, res);
      }
    });
  };
}
//# sourceMappingURL=assets.js.map